#! /usr/bin/env python
"""
    Reads the tail end of the weather_data_raw file coming from the
    get_serial_data.py script, generates summary data and sends the data to
    the CWOP servers.  It's designed to be run every ten minutes via
    cron.

    NOTE: At the moment, the script does not send precipitation data.
    
    Script Copyright (C) 2008, Christopher S. Swingley <cswingle@gmail.com>
    and licensed under the terms of the GNU General Public License v.3.

"""

import math
import datetime
import time
import socket
import telnetlib

SEND = True # Send to CWOP
wspeed_offset = 1.0 # Added and subtracted to speed to get a proper wdir average
                    # Don't use for real u and v vectors or wspeed

def tail(filepath, window=20):
    f = open(filepath, 'rU')
    f.seek(0, 2)
    bytes= f.tell()
    size= window
    block= -1
    while size > 0 and bytes+block*1024  > 0:
        f.seek(block*1024, 2) # from the end!
        data= f.read(1024)
        linesFound= data.count('\n')
        size -= linesFound
        block -= 1
    f.seek(block*1024, 2)
    f.readline() # find a newline
    lastBlocks= list(f.readlines())
    f.close()
    return(lastBlocks[-window:])

last_lines = tail('/home/cswingle/weather_data_raw', window=150) # 150 = 5 minutes
tempsum = 0.0
rhsum = 0.0
pressum = 0.0
voltagesum = 0.0
wspdsum = 0.0
usum = 0.0
vsum = 0.0
maxwind = 0.0
secsum = 0
count = 0
for line in last_lines:
    error = False
    try:
        (cmd,date,tme,temp,rh,pres,wdir,wsp,wmax,precip,voltage,chill,checksum) = line.split(',')
    except:
        error = True
        print "Error reading line: %s" % line
    if not error:
        (temp, rh, pres, wdir, wsp, wmax, precip, voltage, chill) = map(float, (temp, rh, pres, wdir, wsp, wmax, precip, voltage, chill))
        tempsum += temp
        rhsum += rh
        pressum += pres
        voltagesum += voltage
        wspdsum += wsp
        if wsp > maxwind:
            maxwind = wsp
        usum += (-1 * (wsp + wspeed_offset) * math.sin(math.radians(wdir)))
        vsum += (-1 * (wsp + wspeed_offset) * math.cos(math.radians(wdir)))
        count += 1
        (month, day) = date.split('/')
        (hour, minute, second) = tme.split(':')
        (month, day, hour, minute, second) = map(int, (month, day, hour, minute, second))
        dt = datetime.datetime(2008, month, day, hour, minute, second)
        sec = int(time.mktime(dt.timetuple()))
        secsum += sec

if count > 0:
    temp = tempsum / count
    rh = rhsum / count
    pres = pressum / count
    presmb = pres * 33.864
    voltage = voltagesum / count
    u = usum / count
    v = vsum / count
    wspd = wspdsum / count
    wdir = math.degrees(math.atan2(-u, -v))
    if wdir < 0.0:
        wdir = 360.0 + wdir
    secavg = time.gmtime(secsum / count)
    (yr, mo, dy, hr, mi, se) = secavg[:6]
    dt = datetime.datetime(yr, mo, dy, hr, mi, se)
    print "user DW1454 pass -1 vers linux-css-mkiii 1.00"
    aprs = "DW1454>APRS,TCPXX*:/%02d%02d%02dz6453.63N/14752.54W_%03d/%03dg%03dt%03dh%02db%05d\n" % (dy, hr, mi, round(wdir), round(wspd), round(maxwind), round(temp), round(rh) % 100, round(presmb*10))
    print aprs[:-1]
    if SEND:
        try:
            tn = telnetlib.Telnet('cwop.aprs.net', 14580)
        except socket.gaierror:
            print "Difficulty contacting cwop.aprs.net.  Waiting 15 seconds for a second attempt."
            time.sleep(15)
            tn = telnetlib.Telnet('cwop.aprs.net', 14580)
        tn.write("user DW1454 pass -1 vers linux-css-mkiii 1.00\n")
        time.sleep(3)
        tn.write(aprs)
        time.sleep(3)
        tn.close()
